#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <glib.h>

#include "../include/string.h"
#include "../include/disk.h"

#include "edv_types.h"
#include "edv_archive_obj.h"
#include "edv_utils.h"


edv_archive_object_struct *EDVArchObjectNew(void);
edv_archive_object_struct *EDVArchObjectCopy(
	const edv_archive_object_struct *obj
);
void EDVArchObjectSetPath(
	edv_archive_object_struct *obj, const gchar *path
);
void EDVArchObjectSetPath2(
	edv_archive_object_struct *obj,
	const gchar *parent, const gchar *name
);
void EDVArchObjectDelete(edv_archive_object_struct *obj);


#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)


/*
 *	Create a new Archive Object.
 */
edv_archive_object_struct *EDVArchObjectNew(void)
{
	edv_archive_object_struct *obj = EDV_ARCHIVE_OBJECT(
	    g_malloc0(sizeof(edv_archive_object_struct))
	);
	obj->type = EDV_OBJECT_TYPE_FILE;
	obj->name = NULL;
	obj->full_path = NULL;
	obj->link_target = NULL;
	obj->permissions = 0x00000000;
	obj->access_time = 0l;
	obj->modify_time = 0l;
	obj->change_time = 0l;
	obj->owner_name = NULL;
	obj->group_name = NULL;
	obj->size = 0l;
	obj->compressed_size = 0l;
	obj->device_type = 0;
	obj->encryption = NULL;
	obj->compression_ratio = 0.0f;
	obj->method = NULL;
	obj->crc = NULL;

	return(obj);
}


/*
 *	Coppies the Archive Object.
 */
edv_archive_object_struct *EDVArchObjectCopy(
	const edv_archive_object_struct *obj
)
{
	edv_archive_object_struct *tar;

	if(obj == NULL)
	    return(NULL);

	tar = EDVArchObjectNew();
	if(tar == NULL)
	    return(NULL);

	/* Begin copying values */
	tar->type = obj->type;

	tar->name = STRDUP(obj->name);
	tar->full_path = STRDUP(obj->full_path);
	tar->link_target = STRDUP(obj->link_target);

	tar->permissions = obj->permissions;
	tar->access_time = obj->access_time;
	tar->modify_time = obj->modify_time;
	tar->change_time = obj->change_time;

	tar->owner_name = STRDUP(obj->owner_name);
	tar->group_name = STRDUP(obj->group_name);

	tar->size = obj->size;
	tar->compressed_size = obj->compressed_size;

	tar->device_type = obj->device_type;

	tar->encryption = STRDUP(obj->encryption);
	tar->compression_ratio = obj->compression_ratio;
	tar->method = STRDUP(obj->method);
	tar->crc = STRDUP(obj->crc);

	return(tar);
}

/*
 *	Deletes the Archive Object.
 */
void EDVArchObjectDelete(edv_archive_object_struct *obj)
{
	if(obj == NULL)
	    return;

	g_free(obj->name);
	g_free(obj->full_path);

	g_free(obj->link_target);

	g_free(obj->owner_name);
	g_free(obj->group_name);

	g_free(obj->encryption);
	g_free(obj->method);
	g_free(obj->crc);

	g_free(obj);
}
